@@ -1,7 +1,9 @@
 """Test utilities."""
 import atexit
 from contextlib import ExitStack
+import copy
 from importlib import reload as reload_module
+import importlib.resources
 import io
 import logging
 import multiprocessing
@@ -37,11 +39,6 @@ from certbot.compat import os
 from certbot.display import util as display_util
 from certbot.plugins import common
 
-if sys.version_info >= (3, 9):  # pragma: no cover
-    import importlib.resources as importlib_resources
-else:  # pragma: no cover
-    import importlib_resources
-
 
 class DummyInstaller(common.Installer):
     """Dummy installer plugin for test purpose."""
@@ -83,14 +80,14 @@ def vector_path(*names: str) -> str:
     """Path to a test vector."""
     _file_manager = ExitStack()
     atexit.register(_file_manager.close)
-    vector_ref = importlib_resources.files(__package__).joinpath('testdata', *names)
-    path = _file_manager.enter_context(importlib_resources.as_file(vector_ref))
+    vector_ref = importlib.resources.files(__package__).joinpath('testdata', *names)
+    path = _file_manager.enter_context(importlib.resources.as_file(vector_ref))
     return str(path)
 
 
 def load_vector(*names: str) -> bytes:
     """Load contents of a test vector."""
-    vector_ref = importlib_resources.files(__package__).joinpath('testdata', *names)
+    vector_ref = importlib.resources.files(__package__).joinpath('testdata', *names)
     data = vector_ref.read_bytes()
     # Try at most to convert CRLF to LF when data is text
     try:
@@ -403,7 +400,8 @@ class ConfigTestCase(TempDirTestCase):
     def setUp(self) -> None:
         super().setUp()
         self.config = configuration.NamespaceConfig(
-            mock.MagicMock(**constants.CLI_DEFAULTS),
+            # We make a copy here so any mutable values from CLI_DEFAULTS do not get modified.
+            mock.MagicMock(**copy.deepcopy(constants.CLI_DEFAULTS)),
         )
         self.config.set_argument_sources({})
         self.config.namespace.verb = "certonly"
